"""
this file/module intended to serve as a temporary storage for variables <br>
those are exported from other parts <br>

How it works: <br>
1) A list variable is created and named as myvar <br>
2) To access the variable from other parts, issue the following command <br>
>> from scisuit.vars import myvar <br>
                                          <br>
now myvar can be used as a variable.   <br>
                                          <br>

P.S: Needless to say, myvar will not be visible as a text

"""